//Global Variables
// Define the pin numbers for the rows
// Note that ROW_3 and ROW_4 are different for the large 8x8 matrix and the Arduino UNO shield!
// For the shield ROW_3 is pin 13 and ROW_4 is pin 12
// For the large 8x8 matrix ROW_3 is pin 12 and ROW_4 is pin 13
#define ROW_1       8
#define ROW_2      11
#define ROW_3      13
#define ROW_4      12
#define ROW_5      A0
#define ROW_6      A1
#define ROW_7      A2
#define ROW_8      A3

// Define the pin numbers for the columns
#define COL_1     7
#define COL_2     6
#define COL_3     5
#define COL_4     4
#define COL_5     3
#define COL_6     2
#define COL_7     1
#define COL_8     0

#define BUZZER    10
#define MIC       A5

// Define number of columns and rows
#define SIZE_COLS 8
#define SIZE_ROWS 8

int rows[SIZE_ROWS] = {ROW_1, ROW_2, ROW_3, ROW_4, ROW_5, ROW_6, ROW_7, ROW_8};
int cols[SIZE_COLS] = {COL_1, COL_2, COL_3, COL_4, COL_5, COL_6, COL_7, COL_8};

long loopCount;

#define SAMPLES   256
int sampleBuffer[SAMPLES];

 
#define MIN_P2P_DELAY    0
#define MAX_P2P_DELAY   1000
#define MIN_P2P_HEIGHT   9
 
void setup(){
  for(int i=0; i < SIZE_ROWS; i++){
    pinMode(rows[i], OUTPUT);
    // To turn OFF all rows, set them to HIGH
    digitalWrite(rows[i], HIGH);
  }
  for(int i=0; i < SAMPLES; i++){
    pinMode(cols[i], OUTPUT);
    // To turn OFF all columns, set them to LOW
    digitalWrite(cols[i], LOW);
  }

  // Set ADC sampling rate
  // Prescale   ADPS2 ADPS1 ADPS0   Clock freq (MHz)  Sampling rate (KHz)
  //   2   0 0 1       8     615
  //   4   0 1 0       4     307
  //   8   0 1 1       2     153
  //  16   1 0 0       1    76.8
  //  32   1 0 1     0.5    38.4
  //  64   1 1 0    0.25    19.2
  // 128   1 1 1   0.125     9.6   (default)
  
  // set prescale to 16
  bitSet(ADCSRA,ADPS2);
  bitSet(ADCSRA,ADPS1);
  bitSet(ADCSRA,ADPS0);  

  Serial.begin(115200);                   //setup of Serial module, 115200 bits/second   
  loopCount = 0;

  delay(1000); // Give ADC ref voltage time to settle

  //analogReference(INTERNAL);// 1.1V

}
 
void loop(){

  for(int i=0; i < SAMPLES; i++){
    sampleBuffer[i] = (analogRead(MIC)>>2)<<0;    //read the sensor value using ADC and divide by 4 through bit shift
  }

  char goUpDown = 0;
  int peaks = 0;
  int lastChange = 0;
  int lastPeak = -999;
  int peakFound = 0;
  if(loopCount == 0){
    for(int i = 0; i < SAMPLES; i++){
      peakFound = 0;
      if(i > 0){
        if(sampleBuffer[i-1] < sampleBuffer[i] && goUpDown < 1){
          if(i - lastChange > MIN_P2P_DELAY){
            if(abs(sampleBuffer[i] - lastPeak) > MIN_P2P_HEIGHT){
              if(i - lastChange < MAX_P2P_DELAY){
                peaks++;
                peakFound = 1;
              }
            }
          }
          lastPeak = sampleBuffer[i];
          lastChange = i;
          goUpDown = 1;
        }
        if(sampleBuffer[i-1] > sampleBuffer[i] && goUpDown > -1){
          if(i - lastChange > MIN_P2P_DELAY){
            if(abs(sampleBuffer[i] - lastPeak) > MIN_P2P_HEIGHT){
              if(i - lastChange < MAX_P2P_DELAY){
                peaks++;
                peakFound = 1;
              }
            }
          }
          lastPeak = sampleBuffer[i];
          lastChange = i;
          goUpDown = -1;
        }        
      }
      Serial.print("Original ;");
      Serial.print(sampleBuffer[i]);
      Serial.print(";");
      Serial.print(peakFound);
      Serial.print(";\n");
    }
    Serial.print("Peaks = ;");
    Serial.print(peaks);
    Serial.print("\n");
  }

  if(analogRead(MIC) > 750){
     digitalWrite(cols[1], HIGH);
     digitalWrite(rows[1], LOW);
     //delay(1);
  }
  else{
     digitalWrite(cols[1], LOW);
     digitalWrite(rows[1], HIGH);    
  }
  loopCount++;
   
}
