#! /usr/bin/perl

# Install GD graphics package needed for this script with:
# sudo apt-get install libgd-gd2-perl
# sudo apt-get install -y libdevice-serialport-perl

use strict;
use warnings;
use GD::Simple;
use Device::SerialPort;

my $port = Device::SerialPort->new("/dev/ttyACM0");
 
# create a new image (width, height)
my $img = GD::Simple->new(1920, 1080);


$port->baudrate(115200); # Configure this to match your device
$port->databits(8);
$port->parity("none");
$port->stopbits(1);

$port->write("\nBegin perl serial listener\n");

$img->bgcolor('red');
$img->fgcolor('green');

my $start_time = time();
my $X = 0;
my $X_old = 0;
my $X_step = 1920 / 256;
my $Y_scale = 1080/256;
my $Y = 0;
my $Y_old = -9999;
my $peakFound = 0;
my $value_char = "";
my $peaks = 0;


$img->penSize(5,5);
while (time()-$start_time < 5) {
  my $char = $port->lookfor();
  if ($char){
    if(index($char, "Peaks =")<0){
      #print $char . "\n";
      $char = substr($char, index($char,";")+1);
      $value_char = substr($char, 0, index($char,";"));
      #print $char . "  ";
      $char = substr($char, index($char,";")+1);
      #print $char . "\n";
      $peakFound = substr($char, 0, index($char,";"));
      print "Received character: $value_char, peakFound = $peakFound \n";
      if(length($value_char) > 0){
        $Y = int($value_char) * $Y_scale;
        if($Y_old != -9999){
          $img->moveTo($X_old, $Y_old);
          $img->lineTo($X, $Y);
        }
        if($peakFound == 1999){
          $img->moveTo($X_old, 0);
          $img->lineTo($X_old, 1920);
          $peaks++;
        }
        $Y_old = $Y;
        $X_old = $X;
        $X += $X_step;
      }
    }
    else{
      print "Received character: $char \n";
    }
  }
#  $port->lookclear; # needed to prevent blocking
  #sleep (0.1);
}

# Writing to the serial port
# $port->write("This message going out over serial");

$img->fgcolor('red');
$img->moveTo(1920/2, 100);
$img->font('Times:italic');
$img->fontsize(38);
#$img->string("Peaks = $peaks"); 

# draw a red rectangle with blue borders
#$img->bgcolor('red');
#$img->fgcolor('blue');
#$img->rectangle(10, 10, 50, 50); # (top_left_x, top_left_y, bottom_right_x, bottom_right_y)
                                 # ($x1, $y1, $x2, $y2)
 
$img->bgcolor('green');
$img->fgcolor('red');

#$img->moveTo(0,1080/2);
#$img->lineTo(1920, 1080/2);

#$img->moveTo(1920/2,0);
#$img->lineTo(1920/2, 1080);

#$img->fgcolor('green');
#$img->moveTo(0,1080/2+5*$Y_scale);
#$img->lineTo(1920, 1080/2+5*$Y_scale);

#$img->moveTo(1920/2+2*$X_step,0);
#$img->lineTo(1920/2+2*$X_step, 1080);

#$img->moveTo(1920/2+10*$X_step,0);
#$img->lineTo(1920/2+10*$X_step, 1080);

#$img->rectangle(40, 50, 60, 70); # (top_left_x, top_left_y, bottom_right_x, bottom_right_y)
                                 # ($x1, $y1, $x2, $y2)
# convert into png data
open my $out, '>', 'img.png' or die;
binmode $out;
print $out $img->png;


